@@ -275,6 +275,22 @@
 /**
  * @page history History
  *
+ * @section v1011 10:1:1
+ *
+ *   - Migrate usages of POSIX regular expressions (<regex.h>) to the C++11
+ *    regex library (<regex>).
+ *   - Wrong error message is printed when inotify event queue overflows.
+ *
+ * @section v1001 10:0:1
+ *
+ *   - Fix C99 compatibility in cevent.h by not implying enum.
+ *   - Free session memory.
+ *   - Fix segmentation fault when starting monitor.
+ *   - Add fsw_is_running() function to the C API to check that a monitor is
+ *     running.
+ *   - Fix stop sequence in fsw::fsevents_monitor::run() and in
+ *     fsw::fsevents_monitor::on_stop().
+ *
  * @section v900 9:0:0
  *
  *   - Add fsw::monitor_filter::read_from_file() to load filters from a file.
@@ -674,6 +690,16 @@ FSW_STATUS fsw_add_filter(const FSW_HANDLE handle,
   return fsw_set_last_error(FSW_OK);
 }
 
+bool fsw_is_running(const FSW_HANDLE handle)
+{
+  FSW_SESSION *session = get_session(handle);
+
+  if (!session->monitor)
+    return false;
+
+  return session->monitor->is_running();
+}
+
 FSW_STATUS fsw_start_monitor(const FSW_HANDLE handle)
 {
   try
@@ -681,7 +707,15 @@ FSW_STATUS fsw_start_monitor(const FSW_HANDLE handle)
     FSW_SESSION *session = get_session(handle);
 
     if (!session->monitor)
-      create_monitor(handle, session->type);
+    {
+      FSW_STATUS ret = create_monitor(handle, session->type);
+
+      if (ret != FSW_OK)
+        return fsw_set_last_error(ret);
+    }
+
+    if (session->monitor == nullptr) // create_monitor returned OK, but monitor were not created
+      return fsw_set_last_error(FSW_ERR_UNKNOWN_MONITOR_TYPE);
 
     if (session->monitor->is_running())
       return fsw_set_last_error(int(FSW_ERR_MONITOR_ALREADY_RUNNING));
@@ -758,6 +792,8 @@ FSW_STATUS fsw_destroy_session(const FSW_HANDLE handle)
       }
       delete session->monitor;
     }
+
+    delete session;
   }
   catch (int error)
   {
